import { ClientData, Locations, State } from "./types";
import { db } from "./database";
import { getHumanDate, getIsWeekend } from "./utils";
import { formatDate } from "./utils";

/** Vrátí dnešní datum, případně fiktivní datum pro účely vývoje a testování. */
function getToday(): Date {
    if (process.env.MOCK_DATA) {
        return new Date('2023-05-31');
    }
    return new Date();
}

/** Vrátí "prázdná" (implicitní) data, pokud ještě nikdo nehlasoval. */
function getEmptyData(): ClientData {
    return { date: getHumanDate(getToday()), isWeekend: getIsWeekend(getToday()), choices: {} };
}

/**
 * Vrátí veškerá klientská data pro aktuální den.
 */
export function getData(): ClientData {
    const data = db.get(formatDate(getToday())) || getEmptyData();
    return data;
}

/**
 * Vytvoří pizza day pro aktuální den a vrátí data pro klienta.
 */
export function createPizzaDay(creator: string): ClientData {
    initIfNeeded();
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den již existuje");
    }
    const data: ClientData = { pizzaDay: { state: State.CREATED, creator, orders: [] }, ...clientData };
    db.set(today, data);
    return data;
}

/**
 * Smaže pizza day pro aktuální den.
 */
export function deletePizzaDay(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Login uživatele se neshoduje se zakladatelem Pizza Day");
    }
    db.delete(today);
}

export function initIfNeeded() {
    const today = formatDate(getToday());
    if (!db.has(today)) {
        db.set(today, getEmptyData());
    }
}

export function removeChoice(login: string, data: ClientData) {
    for (let key of Object.keys(data.choices)) {
        if (data.choices[key] && data.choices[key].includes(login)) {
            const index = data.choices[key].indexOf(login);
            data.choices[key].splice(index, 1);
            if (data.choices[key].length == 0) {
                delete data.choices[key];
            }
        }
    }
    return data;
}

export function updateChoice(login: string, choice: Locations | null) {
    initIfNeeded();
    const today = formatDate(getToday());
    let data: ClientData = db.get(today);
    data = removeChoice(login, data);
    if (choice !== null) {
        if (!data.choices?.[choice]) {
            data.choices[choice] = [];
        }
        data.choices[choice].push(login);
    }
    db.set(today, data);
    return data;
}